@@ -16,26 +16,22 @@
 
 #include <grpc/support/port_platform.h>
 
-#include <string.h>
-
 #include <cstdint>
 #include <memory>
 #include <random>
 #include <string>
+#include <type_traits>
 #include <utility>
 
-#include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
-#include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
 #include "absl/types/optional.h"
 
-#include <grpc/grpc.h>
-#include <grpc/grpc_security.h>
 #include <grpc/support/log.h>
 
+#include "src/core/ext/gcp/metadata_query.h"
 #include "src/core/ext/xds/xds_bootstrap.h"
 #include "src/core/ext/xds/xds_client_grpc.h"
 #include "src/core/lib/channel/channel_args.h"
@@ -44,21 +40,16 @@
 #include "src/core/lib/gprpp/env.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/gprpp/time.h"
 #include "src/core/lib/gprpp/work_serializer.h"
-#include "src/core/lib/http/httpcli.h"
-#include "src/core/lib/http/parser.h"
-#include "src/core/lib/iomgr/closure.h"
-#include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/polling_entity.h"
 #include "src/core/lib/json/json.h"
+#include "src/core/lib/json/json_writer.h"
 #include "src/core/lib/resolver/resolver.h"
 #include "src/core/lib/resolver/resolver_factory.h"
 #include "src/core/lib/resolver/resolver_registry.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
 #include "src/core/lib/security/credentials/alts/check_gcp_environment.h"
-#include "src/core/lib/security/credentials/credentials.h"
 #include "src/core/lib/uri/uri_parser.h"
 
 namespace grpc_core {
@@ -77,53 +68,6 @@ class GoogleCloud2ProdResolver : public Resolver {
   void ShutdownLocked() override;
 
  private:
-  // Represents an HTTP request to the metadata server.
-  class MetadataQuery : public InternallyRefCounted<MetadataQuery> {
-   public:
-    MetadataQuery(RefCountedPtr<GoogleCloud2ProdResolver> resolver,
-                  const char* path, grpc_polling_entity* pollent);
-    ~MetadataQuery() override;
-
-    void Orphan() override;
-
-   private:
-    static void OnHttpRequestDone(void* arg, grpc_error_handle error);
-
-    // If error is not absl::OkStatus(), then it's not safe to look at response.
-    virtual void OnDone(GoogleCloud2ProdResolver* resolver,
-                        const grpc_http_response* response,
-                        grpc_error_handle error) = 0;
-
-    RefCountedPtr<GoogleCloud2ProdResolver> resolver_;
-    OrphanablePtr<HttpRequest> http_request_;
-    grpc_http_response response_;
-    grpc_closure on_done_;
-  };
-
-  // A metadata server query to get the zone.
-  class ZoneQuery : public MetadataQuery {
-   public:
-    ZoneQuery(RefCountedPtr<GoogleCloud2ProdResolver> resolver,
-              grpc_polling_entity* pollent);
-
-   private:
-    void OnDone(GoogleCloud2ProdResolver* resolver,
-                const grpc_http_response* response,
-                grpc_error_handle error) override;
-  };
-
-  // A metadata server query to get the IPv6 address.
-  class IPv6Query : public MetadataQuery {
-   public:
-    IPv6Query(RefCountedPtr<GoogleCloud2ProdResolver> resolver,
-              grpc_polling_entity* pollent);
-
-   private:
-    void OnDone(GoogleCloud2ProdResolver* resolver,
-                const grpc_http_response* response,
-                grpc_error_handle error) override;
-  };
-
   void ZoneQueryDone(std::string zone);
   void IPv6QueryDone(bool ipv6_supported);
   void StartXdsResolver();
@@ -136,128 +80,13 @@ class GoogleCloud2ProdResolver : public Resolver {
   std::string metadata_server_name_ = "metadata.google.internal.";
   bool shutdown_ = false;
 
-  OrphanablePtr<ZoneQuery> zone_query_;
+  OrphanablePtr<MetadataQuery> zone_query_;
   absl::optional<std::string> zone_;
 
-  OrphanablePtr<IPv6Query> ipv6_query_;
+  OrphanablePtr<MetadataQuery> ipv6_query_;
   absl::optional<bool> supports_ipv6_;
 };
 
-//
-// GoogleCloud2ProdResolver::MetadataQuery
-//
-
-GoogleCloud2ProdResolver::MetadataQuery::MetadataQuery(
-    RefCountedPtr<GoogleCloud2ProdResolver> resolver, const char* path,
-    grpc_polling_entity* pollent)
-    : resolver_(std::move(resolver)) {
-  // Start HTTP request.
-  GRPC_CLOSURE_INIT(&on_done_, OnHttpRequestDone, this, nullptr);
-  Ref().release();  // Ref held by callback.
-  grpc_http_request request;
-  memset(&request, 0, sizeof(grpc_http_request));
-  grpc_http_header header = {const_cast<char*>("Metadata-Flavor"),
-                             const_cast<char*>("Google")};
-  request.hdr_count = 1;
-  request.hdrs = &header;
-  auto uri = URI::Create("http", resolver_->metadata_server_name_, path,
-                         {} /* query params */, "" /* fragment */);
-  GPR_ASSERT(uri.ok());  // params are hardcoded
-  grpc_arg resource_quota_arg = grpc_channel_arg_pointer_create(
-      const_cast<char*>(GRPC_ARG_RESOURCE_QUOTA),
-      resolver_->resource_quota_.get(), grpc_resource_quota_arg_vtable());
-  grpc_channel_args args = {1, &resource_quota_arg};
-  http_request_ =
-      HttpRequest::Get(std::move(*uri), &args, pollent, &request,
-                       Timestamp::Now() + Duration::Seconds(10),  // 10s timeout
-                       &on_done_, &response_,
-                       RefCountedPtr<grpc_channel_credentials>(
-                           grpc_insecure_credentials_create()));
-  http_request_->Start();
-}
-
-GoogleCloud2ProdResolver::MetadataQuery::~MetadataQuery() {
-  grpc_http_response_destroy(&response_);
-}
-
-void GoogleCloud2ProdResolver::MetadataQuery::Orphan() {
-  http_request_.reset();
-  Unref();
-}
-
-void GoogleCloud2ProdResolver::MetadataQuery::OnHttpRequestDone(
-    void* arg, grpc_error_handle error) {
-  auto* self = static_cast<MetadataQuery*>(arg);
-  // Hop back into WorkSerializer to call OnDone().
-  // Note: We implicitly pass our ref to the callback here.
-  self->resolver_->work_serializer_->Run(
-      [self, error]() {
-        self->OnDone(self->resolver_.get(), &self->response_, error);
-        self->Unref();
-      },
-      DEBUG_LOCATION);
-}
-
-//
-// GoogleCloud2ProdResolver::ZoneQuery
-//
-
-GoogleCloud2ProdResolver::ZoneQuery::ZoneQuery(
-    RefCountedPtr<GoogleCloud2ProdResolver> resolver,
-    grpc_polling_entity* pollent)
-    : MetadataQuery(std::move(resolver), "/computeMetadata/v1/instance/zone",
-                    pollent) {}
-
-void GoogleCloud2ProdResolver::ZoneQuery::OnDone(
-    GoogleCloud2ProdResolver* resolver, const grpc_http_response* response,
-    grpc_error_handle error) {
-  absl::StatusOr<std::string> zone;
-  if (!error.ok()) {
-    zone = absl::UnknownError(absl::StrCat(
-        "error fetching zone from metadata server: ", StatusToString(error)));
-  } else if (response->status != 200) {
-    zone = absl::UnknownError(absl::StrFormat(
-        "zone query received non-200 status: %d", response->status));
-  } else {
-    absl::string_view body(response->body, response->body_length);
-    size_t i = body.find_last_of('/');
-    if (i == body.npos) {
-      zone = absl::UnknownError(
-          absl::StrCat("could not parse zone from metadata server: ", body));
-    } else {
-      zone = std::string(body.substr(i + 1));
-    }
-  }
-  if (!zone.ok()) {
-    gpr_log(GPR_ERROR, "zone query failed: %s",
-            zone.status().ToString().c_str());
-    resolver->ZoneQueryDone("");
-  } else {
-    resolver->ZoneQueryDone(std::move(*zone));
-  }
-}
-
-//
-// GoogleCloud2ProdResolver::IPv6Query
-//
-
-GoogleCloud2ProdResolver::IPv6Query::IPv6Query(
-    RefCountedPtr<GoogleCloud2ProdResolver> resolver,
-    grpc_polling_entity* pollent)
-    : MetadataQuery(std::move(resolver),
-                    "/computeMetadata/v1/instance/network-interfaces/0/ipv6s",
-                    pollent) {}
-
-void GoogleCloud2ProdResolver::IPv6Query::OnDone(
-    GoogleCloud2ProdResolver* resolver, const grpc_http_response* response,
-    grpc_error_handle error) {
-  if (!error.ok()) {
-    gpr_log(GPR_ERROR, "error fetching IPv6 address from metadata server: %s",
-            StatusToString(error).c_str());
-  }
-  resolver->IPv6QueryDone(error.ok() && response->status == 200);
-}
-
 //
 // GoogleCloud2ProdResolver
 //
@@ -321,8 +150,33 @@ void GoogleCloud2ProdResolver::StartLocked() {
     return;
   }
   // Using xDS.  Start metadata server queries.
-  zone_query_ = MakeOrphanable<ZoneQuery>(Ref(), &pollent_);
-  ipv6_query_ = MakeOrphanable<IPv6Query>(Ref(), &pollent_);
+  zone_query_ = MakeOrphanable<MetadataQuery>(
+      metadata_server_name_, std::string(MetadataQuery::kZoneAttribute),
+      &pollent_,
+      [resolver = static_cast<RefCountedPtr<GoogleCloud2ProdResolver>>(Ref())](
+          std::string /* attribute */,
+          absl::StatusOr<std::string> result) mutable {
+        resolver->work_serializer_->Run(
+            [resolver, result = std::move(result)]() mutable {
+              resolver->ZoneQueryDone(result.ok() ? std::move(result).value()
+                                                  : "");
+            },
+            DEBUG_LOCATION);
+      },
+      Duration::Seconds(10));
+  ipv6_query_ = MakeOrphanable<MetadataQuery>(
+      metadata_server_name_, std::string(MetadataQuery::kIPv6Attribute),
+      &pollent_,
+      [resolver = static_cast<RefCountedPtr<GoogleCloud2ProdResolver>>(Ref())](
+          std::string /* attribute */,
+          absl::StatusOr<std::string> result) mutable {
+        resolver->work_serializer_->Run(
+            [resolver, result = std::move(result)]() {
+              resolver->IPv6QueryDone(result.ok());
+            },
+            DEBUG_LOCATION);
+      },
+      Duration::Seconds(10));
 }
 
 void GoogleCloud2ProdResolver::RequestReresolutionLocked() {
@@ -393,7 +247,7 @@ void GoogleCloud2ProdResolver::StartXdsResolver() {
                    {"type", "google_default"},
                },
            }},
-          {"server_features", Json::Array{"xds_v3"}},
+          {"server_features", Json::Array{"ignore_resource_deletion"}},
       },
   };
   Json bootstrap = Json::Object{
@@ -408,7 +262,7 @@ void GoogleCloud2ProdResolver::StartXdsResolver() {
       {"node", std::move(node)},
   };
   // Inject bootstrap JSON as fallback config.
-  internal::SetXdsFallbackBootstrapConfig(bootstrap.Dump().c_str());
+  internal::SetXdsFallbackBootstrapConfig(JsonDump(bootstrap).c_str());
   // Now start xDS resolver.
   child_resolver_->StartLocked();
 }
