@@ -54,11 +54,11 @@ class ServiceConfigChannelArgChannelData {
  public:
   explicit ServiceConfigChannelArgChannelData(
       const grpc_channel_element_args* args) {
-    const char* service_config_str = grpc_channel_args_find_string(
-        args->channel_args, GRPC_ARG_SERVICE_CONFIG);
-    if (service_config_str != nullptr) {
+    auto service_config_str =
+        args->channel_args.GetOwnedString(GRPC_ARG_SERVICE_CONFIG);
+    if (service_config_str.has_value()) {
       auto service_config = ServiceConfigImpl::Create(
-          ChannelArgs::FromC(args->channel_args), service_config_str);
+          args->channel_args, service_config_str->c_str());
       if (!service_config.ok()) {
         gpr_log(GPR_ERROR, "%s", service_config.status().ToString().c_str());
       } else {
