@@ -22,6 +22,7 @@
 
 #include <functional>
 #include <memory>
+#include <string>
 #include <utility>
 
 #include "absl/base/attributes.h"
@@ -30,18 +31,19 @@
 #include "absl/types/optional.h"
 
 #include <grpc/grpc.h>
+#include <grpc/support/log.h>
 
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_stack.h"
-#include "src/core/lib/promise/context.h"
-#include "src/core/lib/promise/detail/basic_seq.h"
-#include "src/core/lib/promise/latch.h"
+#include "src/core/lib/debug/trace.h"
+#include "src/core/lib/promise/activity.h"
+#include "src/core/lib/promise/map.h"
+#include "src/core/lib/promise/pipe.h"
+#include "src/core/lib/promise/poll.h"
 #include "src/core/lib/promise/promise.h"
-#include "src/core/lib/promise/seq.h"
-#include "src/core/lib/promise/try_concurrently.h"
-#include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/slice/percent_encoding.h"
 #include "src/core/lib/slice/slice.h"
+#include "src/core/lib/surface/call_trace.h"
 #include "src/core/lib/transport/metadata_batch.h"
 
 namespace grpc_core {
@@ -129,24 +131,23 @@ ArenaPromise<ServerMetadataHandle> HttpServerFilter::MakeCallPromise(
     md->Remove(UserAgentMetadata());
   }
 
-  auto* read_latch = GetContext<Arena>()->New<Latch<ServerMetadata*>>();
-  auto* write_latch =
-      std::exchange(call_args.server_initial_metadata, read_latch);
+  call_args.server_initial_metadata->InterceptAndMap(
+      [](ServerMetadataHandle md) {
+        if (grpc_call_trace.enabled()) {
+          gpr_log(GPR_INFO, "%s[http-server] Write metadata",
+                  Activity::current()->DebugTag().c_str());
+        }
+        FilterOutgoingMetadata(md.get());
+        md->Set(HttpStatusMetadata(), 200);
+        md->Set(ContentTypeMetadata(), ContentTypeMetadata::kApplicationGrpc);
+        return md;
+      });
 
-  return TryConcurrently(
-             Seq(next_promise_factory(std::move(call_args)),
-                 [](ServerMetadataHandle md) -> ServerMetadataHandle {
-                   FilterOutgoingMetadata(md.get());
-                   return md;
-                 }))
-      .Push(Seq(read_latch->Wait(), [write_latch](ServerMetadata** md) {
-        FilterOutgoingMetadata(*md);
-        (*md)->Set(HttpStatusMetadata(), 200);
-        (*md)->Set(ContentTypeMetadata(),
-                   ContentTypeMetadata::kApplicationGrpc);
-        write_latch->Set(*md);
-        return absl::OkStatus();
-      }));
+  return Map(next_promise_factory(std::move(call_args)),
+             [](ServerMetadataHandle md) -> ServerMetadataHandle {
+               FilterOutgoingMetadata(md.get());
+               return md;
+             });
 }
 
 absl::StatusOr<HttpServerFilter> HttpServerFilter::Create(
