@@ -22,6 +22,7 @@
 #include <stdint.h>
 
 #include <algorithm>
+#include <initializer_list>
 #include <map>
 #include <utility>
 
@@ -38,15 +39,16 @@
 #include "google/protobuf/struct.upb.h"
 #include "google/protobuf/struct.upbdefs.h"
 #include "google/protobuf/wrappers.upb.h"
-#include "upb/arena.h"
-#include "upb/json_encode.h"
-#include "upb/status.h"
+#include "upb/base/status.h"
+#include "upb/json/encode.h"
+#include "upb/mem/arena.h"
 #include "upb/upb.hpp"
 #include "xds/type/v3/typed_struct.upb.h"
 
 #include "src/core/ext/xds/upb_utils.h"
 #include "src/core/ext/xds/xds_bootstrap_grpc.h"
 #include "src/core/ext/xds/xds_client.h"
+#include "src/core/lib/json/json_reader.h"
 
 namespace grpc_core {
 
@@ -429,7 +431,7 @@ absl::StatusOr<Json> ParseProtobufStructToJson(
   void* buf = upb_Arena_Malloc(context.arena, json_size + 1);
   upb_JsonEncode(resource, msg_def, context.symtab, 0,
                  reinterpret_cast<char*>(buf), json_size + 1, status.ptr());
-  auto json = Json::Parse(reinterpret_cast<char*>(buf));
+  auto json = JsonParse(reinterpret_cast<char*>(buf));
   if (!json.ok()) {
     // This should never happen.
     return absl::InternalError(
@@ -454,7 +456,7 @@ absl::optional<XdsExtension> ExtractXdsExtension(
     ValidationErrors::ScopedField field(errors, ".type_url");
     if (extension.type.empty()) {
       errors->AddError("field not present");
-      return;
+      return false;
     }
     size_t pos = extension.type.rfind('/');
     if (pos == absl::string_view::npos || pos == extension.type.size() - 1) {
@@ -462,9 +464,10 @@ absl::optional<XdsExtension> ExtractXdsExtension(
     } else {
       extension.type = extension.type.substr(pos + 1);
     }
+    return true;
   };
   extension.type = UpbStringToAbsl(google_protobuf_Any_type_url(any));
-  strip_type_prefix();
+  if (!strip_type_prefix()) return absl::nullopt;
   extension.validation_fields.emplace_back(
       errors, absl::StrCat(".value[", extension.type, "]"));
   absl::string_view any_value = UpbStringToAbsl(google_protobuf_Any_value(any));
@@ -478,7 +481,7 @@ absl::optional<XdsExtension> ExtractXdsExtension(
     }
     extension.type =
         UpbStringToAbsl(xds_type_v3_TypedStruct_type_url(typed_struct));
-    strip_type_prefix();
+    if (!strip_type_prefix()) return absl::nullopt;
     extension.validation_fields.emplace_back(
         errors, absl::StrCat(".value[", extension.type, "]"));
     auto* protobuf_struct = xds_type_v3_TypedStruct_value(typed_struct);
