@@ -1,44 +1,29 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
-#ifndef GRPC_CORE_LIB_DEBUG_TRACE_H
-#define GRPC_CORE_LIB_DEBUG_TRACE_H
-
-#if defined(__has_feature)
-#if __has_feature(thread_sanitizer)
-#define GRPC_THREADSAFE_TRACER
-#endif
-#endif
+#ifndef GRPC_SRC_CORE_LIB_DEBUG_TRACE_H
+#define GRPC_SRC_CORE_LIB_DEBUG_TRACE_H
 
 #include <grpc/support/port_platform.h>
 
-#ifdef GRPC_THREADSAFE_TRACER
 #include <atomic>
-#endif
-
-#include "src/core/lib/gprpp/global_config.h"
-#include "src/core/lib/gprpp/memory.h"
-
-GPR_GLOBAL_CONFIG_DECLARE_STRING(grpc_trace);
-
-// TODO(veblush): Remove this deprecated function once codes depending on this
-// function are updated in the internal repo.
-void grpc_tracer_init(const char* env_var_name);
+#include <map>
+#include <string>
 
 void grpc_tracer_init();
 void grpc_tracer_shutdown(void);
@@ -50,6 +35,7 @@ class TraceFlagList {
  public:
   static bool Set(const char* name, bool enabled);
   static void Add(TraceFlag* flag);
+  static void SaveTo(std::map<std::string, bool>& values);
 
  private:
   static void LogAllTracers();
@@ -79,36 +65,22 @@ class TraceFlag {
 // Prefer GRPC_TRACE_FLAG_ENABLED() macro instead of using enabled() directly.
 #define GRPC_USE_TRACERS  // tracers on by default in OSS
 #if defined(GRPC_USE_TRACERS) || !defined(NDEBUG)
-  bool enabled() {
-#ifdef GRPC_THREADSAFE_TRACER
-    return value_.load(std::memory_order_relaxed);
-#else
-    return value_;
-#endif  // GRPC_THREADSAFE_TRACER
-  }
+  bool enabled() { return value_.load(std::memory_order_relaxed); }
 #else
   bool enabled() { return false; }
-#endif /* defined(GRPC_USE_TRACERS) || !defined(NDEBUG) */
+#endif  // defined(GRPC_USE_TRACERS) || !defined(NDEBUG)
 
  private:
   friend void testing::grpc_tracer_enable_flag(TraceFlag* flag);
   friend class TraceFlagList;
 
   void set_enabled(bool enabled) {
-#ifdef GRPC_THREADSAFE_TRACER
     value_.store(enabled, std::memory_order_relaxed);
-#else
-    value_ = enabled;
-#endif
   }
 
   TraceFlag* next_tracer_;
   const char* const name_;
-#ifdef GRPC_THREADSAFE_TRACER
   std::atomic<bool> value_;
-#else
-  bool value_;
-#endif
 };
 
 #define GRPC_TRACE_FLAG_ENABLED(f) GPR_UNLIKELY((f).enabled())
@@ -128,6 +100,15 @@ class DebugOnlyTraceFlag {
 };
 #endif
 
+class SavedTraceFlags {
+ public:
+  SavedTraceFlags();
+  void Restore();
+
+ private:
+  std::map<std::string, bool> values_;
+};
+
 }  // namespace grpc_core
 
-#endif /* GRPC_CORE_LIB_DEBUG_TRACE_H */
+#endif  // GRPC_SRC_CORE_LIB_DEBUG_TRACE_H
