@@ -17,9 +17,9 @@
 #include "src/core/lib/event_engine/posix_engine/ev_poll_posix.h"
 
 #include <stdint.h>
-#include <stdlib.h>
 
 #include <atomic>
+#include <initializer_list>
 #include <list>
 #include <memory>
 #include <utility>
@@ -28,9 +28,10 @@
 #include "absl/functional/any_invocable.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
+#include "absl/strings/str_format.h"
 
 #include <grpc/event_engine/event_engine.h>
-#include <grpc/impl/codegen/gpr_types.h>
+#include <grpc/status.h>
 #include <grpc/support/log.h>
 #include <grpc/support/sync.h>
 #include <grpc/support/time.h>
@@ -38,7 +39,7 @@
 #include "src/core/lib/event_engine/poller.h"
 #include "src/core/lib/event_engine/posix_engine/event_poller.h"
 #include "src/core/lib/event_engine/posix_engine/posix_engine_closure.h"
-#include "src/core/lib/gprpp/memory.h"
+#include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/iomgr/port.h"
 
 #ifdef GRPC_POSIX_SOCKET_EV_POLL
@@ -56,25 +57,19 @@
 #include "src/core/lib/event_engine/posix_engine/wakeup_fd_posix_default.h"
 #include "src/core/lib/event_engine/time_util.h"
 #include "src/core/lib/gprpp/fork.h"
-#include "src/core/lib/gprpp/global_config.h"
+#include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/gprpp/strerror.h"
 #include "src/core/lib/gprpp/sync.h"
 #include "src/core/lib/gprpp/time.h"
 
-GPR_GLOBAL_CONFIG_DECLARE_STRING(grpc_poll_strategy);
-
 static const intptr_t kClosureNotReady = 0;
 static const intptr_t kClosureReady = 1;
 static const int kPollinCheck = POLLIN | POLLHUP | POLLERR;
 static const int kPolloutCheck = POLLOUT | POLLHUP | POLLERR;
 
 namespace grpc_event_engine {
-namespace posix_engine {
+namespace experimental {
 
-using ::grpc_event_engine::experimental::AnyInvocableClosure;
-using ::grpc_event_engine::experimental::EventEngine;
-using ::grpc_event_engine::experimental::Poller;
-using ::grpc_event_engine::posix_engine::WakeupFd;
 using Events = absl::InlinedVector<PollEventHandle*, 5>;
 
 class PollEventHandle : public EventHandle {
@@ -334,7 +329,7 @@ void ResetEventManagerOnFork() {
 // It is possible that GLIBC has epoll but the underlying kernel doesn't.
 // Create epoll_fd to make sure epoll support is available
 bool InitPollPollerPosix() {
-  if (!grpc_event_engine::posix_engine::SupportsWakeupFd()) {
+  if (!grpc_event_engine::experimental::SupportsWakeupFd()) {
     return false;
   }
   if (grpc_core::Fork::Enabled()) {
@@ -377,13 +372,16 @@ void PollEventHandle::OrphanHandle(PosixEngineClosure* on_done, int* release_fd,
       is_shutdown_ = true;
       shutdown_error_ =
           absl::Status(absl::StatusCode::kInternal, "FD Orphaned");
-      // signal read/write closed to OS so that future operations fail.
-      if (!released_) {
-        shutdown(fd_, SHUT_RDWR);
-      }
+      grpc_core::StatusSetInt(&shutdown_error_,
+                              grpc_core::StatusIntProperty::kRpcStatus,
+                              GRPC_STATUS_UNAVAILABLE);
       SetReadyLocked(&read_closure_);
       SetReadyLocked(&write_closure_);
     }
+    // signal read/write closed to OS so that future operations fail.
+    if (!released_) {
+      shutdown(fd_, SHUT_RDWR);
+    }
     if (!IsWatched()) {
       CloseFd();
     } else {
@@ -414,11 +412,10 @@ int PollEventHandle::NotifyOnLocked(PosixEngineClosure** st,
     scheduler_->Run(closure);
     return 1;
   } else {
-    /* upcallptr was set to a different closure.  This is an error! */
-    gpr_log(GPR_ERROR,
-            "User called a notify_on function with a previous callback still "
-            "pending");
-    abort();
+    // upcallptr was set to a different closure.  This is an error!
+    grpc_core::Crash(
+        "User called a notify_on function with a previous callback still "
+        "pending");
   }
   return 0;
 }
@@ -452,8 +449,9 @@ void PollEventHandle::ShutdownHandle(absl::Status why) {
     if (!is_shutdown_) {
       is_shutdown_ = true;
       shutdown_error_ = why;
-      // signal read/write closed to OS so that future operations fail.
-      shutdown(fd_, SHUT_RDWR);
+      grpc_core::StatusSetInt(&shutdown_error_,
+                              grpc_core::StatusIntProperty::kRpcStatus,
+                              GRPC_STATUS_UNAVAILABLE);
       SetReadyLocked(&read_closure_);
       SetReadyLocked(&write_closure_);
     }
@@ -712,22 +710,19 @@ Poller::WorkResult PollPoller::Work(
       // well instead of crashing. This is because the poller::Work is called
       // right after an event enging is constructed. Even if phony poll is
       // expected to be used, we dont want to check for it until some actual
-      // event handles are registered. Otherwise the event engine construction
+      // event handles are registered. Otherwise the EventEngine construction
       // may crash.
       r = poll(pfds, pfd_count, timeout_ms);
     } else {
-      gpr_log(GPR_ERROR,
-              "Attempted a blocking poll when declared non-polling.");
-      GPR_ASSERT(false);
+      grpc_core::Crash("Attempted a blocking poll when declared non-polling.");
     }
 
     if (r <= 0) {
       if (r < 0 && errno != EINTR) {
         // Abort fail here.
-        gpr_log(GPR_ERROR,
-                "(event_engine) PollPoller:%p encountered poll error: %s", this,
-                grpc_core::StrError(errno).c_str());
-        GPR_ASSERT(false);
+        grpc_core::Crash(absl::StrFormat(
+            "(event_engine) PollPoller:%p encountered poll error: %s", this,
+            grpc_core::StrError(errno).c_str()));
       }
 
       for (i = 1; i < pfd_count; i++) {
@@ -844,37 +839,36 @@ PollPoller* MakePollPoller(Scheduler* scheduler, bool use_phony_poll) {
   return nullptr;
 }
 
-}  // namespace posix_engine
+}  // namespace experimental
 }  // namespace grpc_event_engine
 
-#else /* GRPC_POSIX_SOCKET_EV_POLL */
+#else  // GRPC_POSIX_SOCKET_EV_POLL
+
+#include "src/core/lib/gprpp/crash.h"
 
 namespace grpc_event_engine {
-namespace posix_engine {
-
-using ::grpc_event_engine::experimental::EventEngine;
-using ::grpc_event_engine::experimental::Poller;
+namespace experimental {
 
 PollPoller::PollPoller(Scheduler* /* engine */) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
-void PollPoller::Shutdown() { GPR_ASSERT(false && "unimplemented"); }
+void PollPoller::Shutdown() { grpc_core::Crash("unimplemented"); }
 
-PollPoller::~PollPoller() { GPR_ASSERT(false && "unimplemented"); }
+PollPoller::~PollPoller() { grpc_core::Crash("unimplemented"); }
 
 EventHandle* PollPoller::CreateHandle(int /*fd*/, absl::string_view /*name*/,
                                       bool /*track_err*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
 Poller::WorkResult PollPoller::Work(
     EventEngine::Duration /*timeout*/,
     absl::FunctionRef<void()> /*schedule_poll_again*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
-void PollPoller::Kick() { GPR_ASSERT(false && "unimplemented"); }
+void PollPoller::Kick() { grpc_core::Crash("unimplemented"); }
 
 // If GRPC_LINUX_EPOLL is not defined, it means epoll is not available. Return
 // nullptr.
@@ -884,18 +878,18 @@ PollPoller* MakePollPoller(Scheduler* /*scheduler*/,
 }
 
 void PollPoller::KickExternal(bool /*ext*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
 void PollPoller::PollerHandlesListAddHandle(PollEventHandle* /*handle*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
 void PollPoller::PollerHandlesListRemoveHandle(PollEventHandle* /*handle*/) {
-  GPR_ASSERT(false && "unimplemented");
+  grpc_core::Crash("unimplemented");
 }
 
-}  // namespace posix_engine
+}  // namespace experimental
 }  // namespace grpc_event_engine
 
-#endif /* GRPC_POSIX_SOCKET_EV_POLL */
+#endif  // GRPC_POSIX_SOCKET_EV_POLL
