@@ -27,14 +27,15 @@
 #include "src/core/lib/event_engine/windows/iocp.h"
 #include "src/core/lib/event_engine/windows/win_socket.h"
 #include "src/core/lib/gprpp/crash.h"
+#include "src/core/lib/iomgr/error.h"
 
 namespace grpc_event_engine {
 namespace experimental {
 
 IOCP::IOCP(Executor* executor) noexcept
     : executor_(executor),
-      iocp_handle_(CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL,
-                                          (ULONG_PTR)NULL, 0)) {
+      iocp_handle_(CreateIoCompletionPort(INVALID_HANDLE_VALUE, nullptr,
+                                          (ULONG_PTR) nullptr, 0)) {
   GPR_ASSERT(iocp_handle_);
   WSASocketFlagsInit();
 }
@@ -48,17 +49,18 @@ std::unique_ptr<WinSocket> IOCP::Watch(SOCKET socket) {
       reinterpret_cast<HANDLE>(socket), iocp_handle_,
       reinterpret_cast<uintptr_t>(wrapped_socket.get()), 0);
   if (!ret) {
-    char* utf8_message = gpr_format_message(WSAGetLastError());
-    gpr_log(GPR_ERROR, "Unable to add socket to iocp: %s", utf8_message);
-    gpr_free(utf8_message);
-    __debugbreak();
-    abort();
+    grpc_core::Crash(
+        GRPC_WSA_ERROR(WSAGetLastError(), "Unable to add socket to iocp")
+            .ToString());
   }
   GPR_ASSERT(ret == iocp_handle_);
   return wrapped_socket;
 }
 
 void IOCP::Shutdown() {
+  GRPC_EVENT_ENGINE_POLLER_TRACE(
+      "IOCP::%p shutting down. Outstanding kicks: %d", this,
+      outstanding_kicks_.load());
   while (outstanding_kicks_.load() > 0) {
     Work(std::chrono::hours(42), []() {});
   }
@@ -67,56 +69,38 @@ void IOCP::Shutdown() {
 
 Poller::WorkResult IOCP::Work(EventEngine::Duration timeout,
                               absl::FunctionRef<void()> schedule_poll_again) {
-  static const absl::Status kDeadlineExceeded = absl::DeadlineExceededError(
-      absl::StrFormat("IOCP::%p: Received no completions", this));
-  static const absl::Status kKicked =
-      absl::AbortedError(absl::StrFormat("IOCP::%p: Awoken from a kick", this));
   DWORD bytes = 0;
   ULONG_PTR completion_key;
   LPOVERLAPPED overlapped;
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_event_engine_trace)) {
-    gpr_log(GPR_DEBUG, "IOCP::%p doing work", this);
-  }
+  GRPC_EVENT_ENGINE_POLLER_TRACE("IOCP::%p doing work", this);
   BOOL success = GetQueuedCompletionStatus(
       iocp_handle_, &bytes, &completion_key, &overlapped,
       static_cast<DWORD>(Milliseconds(timeout)));
-  if (success == 0 && overlapped == NULL) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_event_engine_trace)) {
-      gpr_log(GPR_DEBUG, "IOCP::%p deadline exceeded", this);
-    }
+  if (success == 0 && overlapped == nullptr) {
+    GRPC_EVENT_ENGINE_POLLER_TRACE("IOCP::%p deadline exceeded", this);
     return Poller::WorkResult::kDeadlineExceeded;
   }
   GPR_ASSERT(completion_key && overlapped);
   if (overlapped == &kick_overlap_) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_event_engine_trace)) {
-      gpr_log(GPR_DEBUG, "IOCP::%p kicked", this);
-    }
+    GRPC_EVENT_ENGINE_POLLER_TRACE("IOCP::%p kicked", this);
     outstanding_kicks_.fetch_sub(1);
     if (completion_key == (ULONG_PTR)&kick_token_) {
       return Poller::WorkResult::kKicked;
     }
     grpc_core::Crash(
-        absl::StrFormat("Unknown custom completion key: %p", completion_key));
+        absl::StrFormat("Unknown custom completion key: %lu", completion_key));
   }
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_event_engine_trace)) {
-    gpr_log(GPR_DEBUG, "IOCP::%p got event on OVERLAPPED::%p", this,
-            overlapped);
-  }
-  WinSocket* socket = reinterpret_cast<WinSocket*>(completion_key);
-  // TODO(hork): move the following logic into the WinSocket impl.
+  GRPC_EVENT_ENGINE_POLLER_TRACE("IOCP::%p got event on OVERLAPPED::%p", this,
+                                 overlapped);
+  // Safety note: socket is guaranteed to exist when managed by a
+  // WindowsEndpoint. If an overlapped event came in, then either a read event
+  // handler is registered, which keeps the socket alive, or the WindowsEndpoint
+  // (which keeps the socket alive) has done an asynchronous WSARecv and is
+  // about to register for notification of an overlapped event.
+  auto* socket = reinterpret_cast<WinSocket*>(completion_key);
   WinSocket::OpState* info = socket->GetOpInfoForOverlapped(overlapped);
   GPR_ASSERT(info != nullptr);
-  if (socket->IsShutdown()) {
-    info->SetError(WSAESHUTDOWN);
-  } else {
-    info->GetOverlappedResult();
-  }
-  if (info->closure() != nullptr) {
-    schedule_poll_again();
-    executor_->Run(info->closure());
-    return Poller::WorkResult::kOk;
-  }
-  // No callback registered. Set ready and return an empty set
+  info->GetOverlappedResult();
   info->SetReady();
   schedule_poll_again();
   return Poller::WorkResult::kOk;
@@ -140,7 +124,7 @@ DWORD IOCP::WSASocketFlagsInit() {
   // versions, see
   // https://msdn.microsoft.com/en-us/library/windows/desktop/ms742212(v=vs.85).aspx
   // for details.
-  SOCKET sock = WSASocket(AF_INET6, SOCK_STREAM, IPPROTO_TCP, NULL, 0,
+  SOCKET sock = WSASocket(AF_INET6, SOCK_STREAM, IPPROTO_TCP, nullptr, 0,
                           wsa_socket_flags | WSA_FLAG_NO_HANDLE_INHERIT);
   if (sock != INVALID_SOCKET) {
     // Windows 7, Windows 2008 R2 with SP1 or later
