@@ -43,6 +43,7 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 
+#include <grpc/byte_buffer.h>
 #include <grpc/event_engine/endpoint_config.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
@@ -50,10 +51,17 @@
 #include <grpc/support/time.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
+#include "src/core/lib/event_engine/default_event_engine.h"
+#include "src/core/lib/event_engine/memory_allocator_factory.h"
+#include "src/core/lib/event_engine/resolved_address_internal.h"
+#include "src/core/lib/event_engine/shim.h"
+#include "src/core/lib/event_engine/tcp_socket_utils.h"
 #include "src/core/lib/gpr/string.h"
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/gprpp/memory.h"
 #include "src/core/lib/gprpp/strerror.h"
+#include "src/core/lib/iomgr/event_engine_shims/closure.h"
+#include "src/core/lib/iomgr/event_engine_shims/endpoint.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/iomgr/resolve_address.h"
 #include "src/core/lib/iomgr/sockaddr.h"
@@ -64,10 +72,121 @@
 #include "src/core/lib/iomgr/tcp_server_utils_posix.h"
 #include "src/core/lib/iomgr/unix_sockets_posix.h"
 #include "src/core/lib/resource_quota/api.h"
+#include "src/core/lib/transport/error_utils.h"
 
 static std::atomic<int64_t> num_dropped_connections{0};
 
 using ::grpc_event_engine::experimental::EndpointConfig;
+using ::grpc_event_engine::experimental::EventEngine;
+using ::grpc_event_engine::experimental::MemoryAllocator;
+using ::grpc_event_engine::experimental::MemoryQuotaBasedMemoryAllocatorFactory;
+using ::grpc_event_engine::experimental::PosixEventEngineWithFdSupport;
+using ::grpc_event_engine::experimental::SliceBuffer;
+
+static grpc_error_handle CreateEventEngineListener(
+    grpc_tcp_server* s, grpc_closure* shutdown_complete,
+    const EndpointConfig& config, grpc_tcp_server** server) {
+  PosixEventEngineWithFdSupport::PosixAcceptCallback accept_cb =
+      [s](int listener_fd, std::unique_ptr<EventEngine::Endpoint> ep,
+          bool is_external, MemoryAllocator /*allocator*/,
+          SliceBuffer* pending_data) {
+        grpc_core::ApplicationCallbackExecCtx app_ctx;
+        grpc_core::ExecCtx exec_ctx;
+        grpc_tcp_server_acceptor* acceptor =
+            static_cast<grpc_tcp_server_acceptor*>(
+                gpr_malloc(sizeof(*acceptor)));
+        acceptor->from_server = s;
+        acceptor->port_index = -1;
+        acceptor->fd_index = -1;
+        if (!is_external) {
+          auto it = s->listen_fd_to_index_map.find(listener_fd);
+          if (it != s->listen_fd_to_index_map.end()) {
+            acceptor->port_index = std::get<0>(it->second);
+            acceptor->fd_index = std::get<1>(it->second);
+          }
+        } else {
+          // External connection handling.
+          grpc_resolved_address addr;
+          memset(&addr, 0, sizeof(addr));
+          addr.len = static_cast<socklen_t>(sizeof(struct sockaddr_storage));
+          // Get the fd of the socket connected to peer.
+          int fd =
+              reinterpret_cast<
+                  grpc_event_engine::experimental::PosixEndpointWithFdSupport*>(
+                  ep.get())
+                  ->GetWrappedFd();
+          if (getpeername(fd, reinterpret_cast<struct sockaddr*>(addr.addr),
+                          &(addr.len)) < 0) {
+            gpr_log(GPR_ERROR, "Failed getpeername: %s",
+                    grpc_core::StrError(errno).c_str());
+            close(fd);
+            return;
+          }
+          (void)grpc_set_socket_no_sigpipe_if_possible(fd);
+          auto addr_uri = grpc_sockaddr_to_uri(&addr);
+          if (!addr_uri.ok()) {
+            gpr_log(GPR_ERROR, "Invalid address: %s",
+                    addr_uri.status().ToString().c_str());
+            return;
+          }
+          if (grpc_tcp_trace.enabled()) {
+            gpr_log(GPR_INFO,
+                    "SERVER_CONNECT: incoming external connection: %s",
+                    addr_uri->c_str());
+          }
+        }
+        grpc_pollset* read_notifier_pollset =
+            (*(s->pollsets))[static_cast<size_t>(gpr_atm_no_barrier_fetch_add(
+                                 &s->next_pollset_to_assign, 1)) %
+                             s->pollsets->size()];
+        acceptor->external_connection = is_external;
+        acceptor->listener_fd = listener_fd;
+        grpc_byte_buffer* buf = nullptr;
+        if (pending_data != nullptr && pending_data->Length() > 0) {
+          buf = grpc_raw_byte_buffer_create(nullptr, 0);
+          grpc_slice_buffer_swap(&buf->data.raw.slice_buffer,
+                                 pending_data->c_slice_buffer());
+          pending_data->Clear();
+        }
+        acceptor->pending_data = buf;
+        s->on_accept_cb(
+            s->on_accept_cb_arg,
+            grpc_event_engine::experimental::grpc_event_engine_endpoint_create(
+                std::move(ep)),
+            read_notifier_pollset, acceptor);
+      };
+  auto on_shutdown_complete_cb =
+      grpc_event_engine::experimental::GrpcClosureToStatusCallback(
+          shutdown_complete);
+  PosixEventEngineWithFdSupport* engine_ptr =
+      reinterpret_cast<PosixEventEngineWithFdSupport*>(
+          config.GetVoidPointer(GRPC_INTERNAL_ARG_EVENT_ENGINE));
+  // Keeps the engine alive for some tests that have not otherwise instantiated
+  // an EventEngine
+  std::shared_ptr<EventEngine> keeper;
+  if (engine_ptr == nullptr) {
+    keeper = grpc_event_engine::experimental::GetDefaultEventEngine();
+    engine_ptr = reinterpret_cast<PosixEventEngineWithFdSupport*>(keeper.get());
+  }
+  auto listener = engine_ptr->CreatePosixListener(
+      std::move(accept_cb),
+      [s, shutdown_complete](absl::Status status) {
+        grpc_event_engine::experimental::RunEventEngineClosure(
+            shutdown_complete, absl_status_to_grpc_error(status));
+        delete s->fd_handler;
+        delete s;
+      },
+      config,
+      std::make_unique<MemoryQuotaBasedMemoryAllocatorFactory>(
+          s->memory_quota));
+  if (!listener.ok()) {
+    delete s;
+    *server = nullptr;
+    return listener.status();
+  }
+  s->ee_listener = std::move(*listener);
+  return absl::OkStatus();
+}
 
 static grpc_error_handle tcp_server_create(grpc_closure* shutdown_complete,
                                            const EndpointConfig& config,
@@ -92,7 +211,11 @@ static grpc_error_handle tcp_server_create(grpc_closure* shutdown_complete,
   s->shutdown = false;
   s->shutdown_starting.head = nullptr;
   s->shutdown_starting.tail = nullptr;
-  s->shutdown_complete = shutdown_complete;
+  if (!grpc_event_engine::experimental::UseEventEngineListener()) {
+    s->shutdown_complete = shutdown_complete;
+  } else {
+    s->shutdown_complete = nullptr;
+  }
   s->on_accept_cb = on_accept_cb;
   s->on_accept_cb_arg = on_accept_cb_arg;
   s->head = nullptr;
@@ -105,7 +228,13 @@ static grpc_error_handle tcp_server_create(grpc_closure* shutdown_complete,
   s->memory_quota = s->options.resource_quota->memory_quota();
   s->pre_allocated_fd = -1;
   gpr_atm_no_barrier_store(&s->next_pollset_to_assign, 0);
+  s->n_bind_ports = 0;
+  new (&s->listen_fd_to_index_map)
+      absl::flat_hash_map<int, std::tuple<int, int>>();
   *server = s;
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    return CreateEventEngineListener(s, shutdown_complete, config, server);
+  }
   return absl::OkStatus();
 }
 
@@ -123,8 +252,14 @@ static void finish_shutdown(grpc_tcp_server* s) {
     s->head = sp->next;
     gpr_free(sp);
   }
-  delete s->fd_handler;
-  delete s;
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    // This will trigger asynchronous execution of the on_shutdown_complete
+    // callback when appropriate. That callback will delete the server
+    s->ee_listener.reset();
+  } else {
+    delete s->fd_handler;
+    delete s;
+  }
 }
 
 static void destroyed_port(void* server, grpc_error_handle /*error*/) {
@@ -423,6 +558,30 @@ static grpc_error_handle clone_port(grpc_tcp_listener* listener,
 static grpc_error_handle tcp_server_add_port(grpc_tcp_server* s,
                                              const grpc_resolved_address* addr,
                                              int* out_port) {
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    gpr_mu_lock(&s->mu);
+    if (s->shutdown_listeners) {
+      gpr_mu_unlock(&s->mu);
+      return absl::UnknownError("Server already shutdown");
+    }
+    int fd_index = 0;
+    auto port = s->ee_listener->BindWithFd(
+        grpc_event_engine::experimental::CreateResolvedAddress(*addr),
+        [s, &fd_index](absl::StatusOr<int> listen_fd) {
+          if (!listen_fd.ok()) {
+            return;
+          }
+          GPR_DEBUG_ASSERT(*listen_fd > 0);
+          s->listen_fd_to_index_map.insert_or_assign(
+              *listen_fd, std::make_tuple(s->n_bind_ports, fd_index++));
+        });
+    if (port.ok()) {
+      s->n_bind_ports++;
+      *out_port = *port;
+    }
+    gpr_mu_unlock(&s->mu);
+    return port.status();
+  }
   GPR_ASSERT(addr->len <= GRPC_MAX_SOCKADDR_SIZE);
   grpc_tcp_listener* sp;
   grpc_resolved_address sockname_temp;
@@ -500,6 +659,17 @@ static grpc_tcp_listener* get_port_index(grpc_tcp_server* s,
 unsigned tcp_server_port_fd_count(grpc_tcp_server* s, unsigned port_index) {
   unsigned num_fds = 0;
   gpr_mu_lock(&s->mu);
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    // This doesn't need to be very fast. Used in tests.
+    for (auto it = s->listen_fd_to_index_map.begin();
+         it != s->listen_fd_to_index_map.end(); it++) {
+      if (std::get<0>(it->second) == static_cast<int>(port_index)) {
+        num_fds++;
+      }
+    }
+    gpr_mu_unlock(&s->mu);
+    return num_fds;
+  }
   grpc_tcp_listener* sp = get_port_index(s, port_index);
   for (; sp; sp = sp->sibling) {
     ++num_fds;
@@ -511,6 +681,19 @@ unsigned tcp_server_port_fd_count(grpc_tcp_server* s, unsigned port_index) {
 static int tcp_server_port_fd(grpc_tcp_server* s, unsigned port_index,
                               unsigned fd_index) {
   gpr_mu_lock(&s->mu);
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    // This doesn't need to be very fast. Used in tests.
+    for (auto it = s->listen_fd_to_index_map.begin();
+         it != s->listen_fd_to_index_map.end(); it++) {
+      if (std::get<0>(it->second) == static_cast<int>(port_index) &&
+          std::get<1>(it->second) == static_cast<int>(fd_index)) {
+        gpr_mu_unlock(&s->mu);
+        return it->first;
+      }
+    }
+    gpr_mu_unlock(&s->mu);
+    return -1;
+  }
   grpc_tcp_listener* sp = get_port_index(s, port_index);
   for (; sp; sp = sp->sibling, --fd_index) {
     if (fd_index == 0) {
@@ -530,6 +713,12 @@ static void tcp_server_start(grpc_tcp_server* s,
   GPR_ASSERT(s->on_accept_cb);
   GPR_ASSERT(s->active_ports == 0);
   s->pollsets = pollsets;
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    GPR_ASSERT(!s->shutdown_listeners);
+    GPR_ASSERT(GRPC_LOG_IF_ERROR("listener_start", s->ee_listener->Start()));
+    gpr_mu_unlock(&s->mu);
+    return;
+  }
   sp = s->head;
   while (sp != nullptr) {
     if (s->so_reuseport && !grpc_is_unix_socket(&sp->addr) &&
@@ -584,7 +773,10 @@ static void tcp_server_unref(grpc_tcp_server* s) {
 static void tcp_server_shutdown_listeners(grpc_tcp_server* s) {
   gpr_mu_lock(&s->mu);
   s->shutdown_listeners = true;
-  // shutdown all fd's
+  if (grpc_event_engine::experimental::UseEventEngineListener()) {
+    s->ee_listener->ShutdownListeningFds();
+  }
+  /* shutdown all fd's */
   if (s->active_ports) {
     grpc_tcp_listener* sp;
     for (sp = s->head; sp; sp = sp->next) {
@@ -611,6 +803,18 @@ class ExternalConnectionHandler : public grpc_core::TcpServerFdHandler {
 
   // TODO(yangg) resolve duplicate code with on_read
   void Handle(int listener_fd, int fd, grpc_byte_buffer* buf) override {
+    if (grpc_event_engine::experimental::UseEventEngineListener()) {
+      grpc_event_engine::experimental::SliceBuffer pending_data;
+      if (buf != nullptr) {
+        pending_data =
+            grpc_event_engine::experimental::SliceBuffer::TakeCSliceBuffer(
+                buf->data.raw.slice_buffer);
+      }
+      GPR_ASSERT(GRPC_LOG_IF_ERROR("listener_handle_external_connection",
+                                   s_->ee_listener->HandleExternalConnection(
+                                       listener_fd, fd, &pending_data)));
+      return;
+    }
     grpc_pollset* read_notifier_pollset;
     grpc_resolved_address addr;
     memset(&addr, 0, sizeof(addr));
