@@ -113,8 +113,34 @@ end
 $CFLAGS << ' -DGRPC_RUBY_WINDOWS_UCRT' if windows_ucrt
 $CFLAGS << ' -I' + File.join(grpc_root, 'include')
 
-ext_export_file = File.join(grpc_root, 'src', 'ruby', 'ext', 'grpc', 'ext-export')
-ext_export_file += '-truffleruby' if RUBY_ENGINE == 'truffleruby'
+def have_ruby_abi_version()
+  return true if RUBY_ENGINE == 'truffleruby'
+  # ruby_abi_version is only available in development versions: https://github.com/ruby/ruby/pull/6231
+  return false if RUBY_PATCHLEVEL >= 0
+
+  m = /(\d+)\.(\d+)/.match(RUBY_VERSION)
+  if m.nil?
+    puts "Failed to parse ruby version: #{RUBY_VERSION}. Assuming ruby_abi_version symbol is NOT present."
+    return false
+  end
+  major = m[1].to_i
+  minor = m[2].to_i
+  if major >= 3 and minor >= 2
+    puts "Ruby version #{RUBY_VERSION} >= 3.2. Assuming ruby_abi_version symbol is present."
+    return true
+  end
+  puts "Ruby version #{RUBY_VERSION} < 3.2. Assuming ruby_abi_version symbol is NOT present."
+  false
+end
+
+def ext_export_filename()
+  name = 'ext-export'
+  name += '-truffleruby' if RUBY_ENGINE == 'truffleruby'
+  name += '-with-ruby-abi-version' if have_ruby_abi_version()
+  name
+end
+
+ext_export_file = File.join(grpc_root, 'src', 'ruby', 'ext', 'grpc', ext_export_filename())
 $LDFLAGS << ' -Wl,--version-script="' + ext_export_file + '.gcc"' if linux
 $LDFLAGS << ' -Wl,-exported_symbols_list,"' + ext_export_file + '.clang"' if apple_toolchain
 
@@ -161,3 +187,22 @@ if grpc_config == 'opt'
   end
   File.rename('Makefile.new', 'Makefile')
 end
+if ENV['GRPC_RUBY_TEST_ONLY_WORKAROUND_MAKE_INSTALL_BUG']
+  # Note: this env var setting is intended to work around a problem observed
+  # with the ginstall command on grpc's macos automated test infrastructure,
+  # and is not  guaranteed to work in the wild.
+  # Also see https://github.com/rake-compiler/rake-compiler/issues/210.
+  puts 'Overriding the generated Makefile install target to use cp'
+  File.open('Makefile.new', 'w') do |o|
+    File.foreach('Makefile') do |i|
+      if i.start_with?('INSTALL_PROG = ')
+        override = 'INSTALL_PROG = cp'
+        puts "Replacing generated Makefile line: |#{i}|, with: |#{override}|"
+        o.puts override
+      else
+        o.puts i
+      end
+    end
+  end
+  File.rename('Makefile.new', 'Makefile')
+end
