@@ -1,27 +1,31 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #ifndef GRPC_TEST_CORE_END2END_CQ_VERIFIER_H
 #define GRPC_TEST_CORE_END2END_CQ_VERIFIER_H
 
+#include <stdint.h>
+
+#include <functional>
 #include <string>
 #include <vector>
 
+#include "absl/functional/any_invocable.h"
 #include "absl/types/variant.h"
 
 #include <grpc/grpc.h>
@@ -46,10 +50,34 @@ class CqVerifier {
   struct AnyStatus {
     bool* result = nullptr;
   };
+  // PerformAction - expect the tag, and run a function based on the result
+  struct PerformAction {
+    std::function<void(bool success)> action;
+  };
+  // MaybePerformAction - run a function if a tag is seen
+  struct MaybePerformAction {
+    std::function<void(bool success)> action;
+  };
 
-  using ExpectedResult = absl::variant<bool, Maybe, AnyStatus>;
+  using ExpectedResult =
+      absl::variant<bool, Maybe, AnyStatus, PerformAction, MaybePerformAction>;
 
-  explicit CqVerifier(grpc_completion_queue* cq);
+  struct Failure {
+    SourceLocation location;
+    std::string message;
+    std::vector<std::string> expected;
+  };
+
+  static void FailUsingGprCrash(const Failure& failure);
+  static void FailUsingGtestFail(const Failure& failure);
+
+  // Allow customizing the failure handler
+  // For legacy tests we should use FailUsingGprCrash (the default)
+  // For gtest based tests we should start migrating to FailUsingGtestFail which
+  // will produce nicer failure messages.
+  explicit CqVerifier(
+      grpc_completion_queue* cq,
+      absl::AnyInvocable<void(Failure)> fail = FailUsingGprCrash);
   ~CqVerifier();
 
   CqVerifier(const CqVerifier&) = delete;
@@ -72,6 +100,9 @@ class CqVerifier {
               SourceLocation location = SourceLocation());
 
   std::string ToString() const;
+  std::vector<std::string> ToStrings() const;
+
+  static void* tag(intptr_t t) { return reinterpret_cast<void*>(t); }
 
  private:
   struct Expectation {
@@ -89,6 +120,7 @@ class CqVerifier {
 
   grpc_completion_queue* const cq_;
   std::vector<Expectation> expectations_;
+  mutable absl::AnyInvocable<void(Failure)> fail_;
 };
 
 }  // namespace grpc_core
@@ -100,4 +132,4 @@ int contains_metadata(grpc_metadata_array* array, const char* key,
 int contains_metadata_slices(grpc_metadata_array* array, grpc_slice key,
                              grpc_slice value);
 
-#endif /* GRPC_TEST_CORE_END2END_CQ_VERIFIER_H */
+#endif  // GRPC_TEST_CORE_END2END_CQ_VERIFIER_H
