@@ -16,27 +16,27 @@
 //
 //
 
+#ifndef GRPC_TEST_CORE_END2END_FIXTURES_LOCAL_UTIL_H
+#define GRPC_TEST_CORE_END2END_FIXTURES_LOCAL_UTIL_H
+
 #include <string>
 
 #include <grpc/grpc.h>
 #include <grpc/grpc_security_constants.h>
 
+#include "src/core/lib/channel/channel_args.h"
 #include "test/core/end2end/end2end_tests.h"
 
-struct grpc_end2end_local_fullstack_fixture_data {
-  std::string localaddr;
+class LocalTestFixture final : public grpc_core::CoreTestFixture {
+ public:
+  LocalTestFixture(std::string localaddr, grpc_local_connect_type type);
+
+ private:
+  grpc_server* MakeServer(const grpc_core::ChannelArgs& args) override;
+  grpc_channel* MakeClient(const grpc_core::ChannelArgs& args) override;
+
+  std::string localaddr_;
+  grpc_local_connect_type type_;
 };
 
-// Utility functions shared by h2_local tests.
-grpc_end2end_test_fixture grpc_end2end_local_chttp2_create_fixture_fullstack();
-
-void grpc_end2end_local_chttp2_init_client_fullstack(
-    grpc_end2end_test_fixture* f, const grpc_channel_args* client_args,
-    grpc_local_connect_type type);
-
-void grpc_end2end_local_chttp2_init_server_fullstack(
-    grpc_end2end_test_fixture* f, const grpc_channel_args* server_args,
-    grpc_local_connect_type type);
-
-void grpc_end2end_local_chttp2_tear_down_fullstack(
-    grpc_end2end_test_fixture* f);
+#endif  // GRPC_TEST_CORE_END2END_FIXTURES_LOCAL_UTIL_H
