@@ -18,10 +18,12 @@
 
 #include "src/core/ext/xds/xds_lb_policy_registry.h"
 
-#include <algorithm>
+#include <stdint.h>
+
 #include <string>
 
 #include <google/protobuf/any.pb.h>
+#include <google/protobuf/duration.pb.h>
 #include <google/protobuf/struct.pb.h>
 #include <google/protobuf/wrappers.pb.h>
 
@@ -29,7 +31,7 @@
 #include "absl/status/statusor.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
-#include "upb/def.hpp"
+#include "upb/reflection/def.hpp"
 #include "upb/upb.hpp"
 
 #include <grpc/grpc.h>
@@ -40,8 +42,10 @@
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/gprpp/validation_errors.h"
+#include "src/core/lib/json/json_writer.h"
 #include "src/core/lib/load_balancing/lb_policy.h"
 #include "src/core/lib/load_balancing/lb_policy_factory.h"
+#include "src/proto/grpc/testing/xds/v3/client_side_weighted_round_robin.pb.h"
 #include "src/proto/grpc/testing/xds/v3/cluster.pb.h"
 #include "src/proto/grpc/testing/xds/v3/extension.pb.h"
 #include "src/proto/grpc/testing/xds/v3/ring_hash.pb.h"
@@ -56,6 +60,8 @@ namespace {
 
 using LoadBalancingPolicyProto =
     ::envoy::config::cluster::v3::LoadBalancingPolicy;
+using ::envoy::extensions::load_balancing_policies::
+    client_side_weighted_round_robin::v3::ClientSideWeightedRoundRobin;
 using ::envoy::extensions::load_balancing_policies::ring_hash::v3::RingHash;
 using ::envoy::extensions::load_balancing_policies::round_robin::v3::RoundRobin;
 using ::envoy::extensions::load_balancing_policies::wrr_locality::v3::
@@ -64,7 +70,8 @@ using ::xds::type::v3::TypedStruct;
 
 // Uses XdsLbPolicyRegistry to convert
 // envoy::config::cluster::v3::LoadBalancingPolicy to gRPC's JSON form.
-absl::StatusOr<std::string> ConvertXdsPolicy(LoadBalancingPolicyProto policy) {
+absl::StatusOr<std::string> ConvertXdsPolicy(
+    const LoadBalancingPolicyProto& policy) {
   std::string serialized_policy = policy.SerializeAsString();
   upb::Arena arena;
   upb::SymbolTable symtab;
@@ -77,9 +84,12 @@ absl::StatusOr<std::string> ConvertXdsPolicy(LoadBalancingPolicyProto policy) {
   ValidationErrors::ScopedField field(&errors, ".load_balancing_policy");
   auto config = XdsLbPolicyRegistry().ConvertXdsLbPolicyConfig(
       context, upb_policy, &errors);
-  if (!errors.ok()) return errors.status("validation errors");
+  if (!errors.ok()) {
+    return errors.status(absl::StatusCode::kInvalidArgument,
+                         "validation errors");
+  }
   EXPECT_EQ(config.size(), 1);
-  return Json{config[0]}.Dump();
+  return JsonDump(Json{config[0]});
 }
 
 // A gRPC LB policy factory for a custom policy.  None of the methods
@@ -115,6 +125,90 @@ TEST(RoundRobin, Basic) {
   EXPECT_EQ(*result, "{\"round_robin\":{}}");
 }
 
+//
+// ClientSideWeightedRoundRobin
+//
+
+TEST(ClientSideWeightedRoundRobinTest, DefaultConfig) {
+  LoadBalancingPolicyProto policy;
+  policy.add_policies()
+      ->mutable_typed_extension_config()
+      ->mutable_typed_config()
+      ->PackFrom(ClientSideWeightedRoundRobin());
+  auto result = ConvertXdsPolicy(policy);
+  ASSERT_TRUE(result.ok()) << result.status();
+  EXPECT_EQ(*result, "{\"weighted_round_robin\":{}}");
+}
+
+TEST(ClientSideWeightedRoundRobinTest, FieldsExplicitlySet) {
+  ClientSideWeightedRoundRobin wrr;
+  wrr.mutable_enable_oob_load_report()->set_value(true);
+  wrr.mutable_oob_reporting_period()->set_seconds(1);
+  wrr.mutable_blackout_period()->set_seconds(2);
+  wrr.mutable_weight_expiration_period()->set_seconds(3);
+  wrr.mutable_weight_update_period()->set_seconds(4);
+  wrr.mutable_error_utilization_penalty()->set_value(5.0);
+  LoadBalancingPolicyProto policy;
+  policy.add_policies()
+      ->mutable_typed_extension_config()
+      ->mutable_typed_config()
+      ->PackFrom(wrr);
+  auto result = ConvertXdsPolicy(policy);
+  ASSERT_TRUE(result.ok()) << result.status();
+  EXPECT_EQ(*result,
+            "{\"weighted_round_robin\":{"
+            "\"blackoutPeriod\":\"2.000000000s\","
+            "\"enableOobLoadReport\":true,"
+            "\"errorUtilizationPenalty\":5.000000,"
+            "\"oobReportingPeriod\":\"1.000000000s\","
+            "\"weightExpirationPeriod\":\"3.000000000s\","
+            "\"weightUpdatePeriod\":\"4.000000000s\""
+            "}}");
+}
+
+TEST(ClientSideWeightedRoundRobinTest, InvalidValues) {
+  ClientSideWeightedRoundRobin wrr;
+  wrr.mutable_oob_reporting_period()->set_seconds(-1);
+  wrr.mutable_blackout_period()->set_seconds(-2);
+  wrr.mutable_weight_expiration_period()->set_seconds(-3);
+  wrr.mutable_weight_update_period()->set_seconds(-4);
+  wrr.mutable_error_utilization_penalty()->set_value(-1);
+  LoadBalancingPolicyProto policy;
+  policy.add_policies()
+      ->mutable_typed_extension_config()
+      ->mutable_typed_config()
+      ->PackFrom(wrr);
+  auto result = ConvertXdsPolicy(policy);
+  EXPECT_EQ(result.status().code(), absl::StatusCode::kInvalidArgument);
+  EXPECT_EQ(result.status().message(),
+            "validation errors: ["
+            "field:load_balancing_policy.policies[0].typed_extension_config"
+            ".typed_config.value[envoy.extensions.load_balancing_policies"
+            ".client_side_weighted_round_robin.v3.ClientSideWeightedRoundRobin]"
+            ".blackout_period.seconds "
+            "error:value must be in the range [0, 315576000000]; "
+            "field:load_balancing_policy.policies[0].typed_extension_config"
+            ".typed_config.value[envoy.extensions.load_balancing_policies"
+            ".client_side_weighted_round_robin.v3.ClientSideWeightedRoundRobin]"
+            ".error_utilization_penalty error:value must be non-negative; "
+            "field:load_balancing_policy.policies[0].typed_extension_config"
+            ".typed_config.value[envoy.extensions.load_balancing_policies"
+            ".client_side_weighted_round_robin.v3.ClientSideWeightedRoundRobin]"
+            ".oob_reporting_period.seconds "
+            "error:value must be in the range [0, 315576000000]; "
+            "field:load_balancing_policy.policies[0].typed_extension_config"
+            ".typed_config.value[envoy.extensions.load_balancing_policies"
+            ".client_side_weighted_round_robin.v3.ClientSideWeightedRoundRobin]"
+            ".weight_expiration_period.seconds "
+            "error:value must be in the range [0, 315576000000]; "
+            "field:load_balancing_policy.policies[0].typed_extension_config"
+            ".typed_config.value[envoy.extensions.load_balancing_policies"
+            ".client_side_weighted_round_robin.v3.ClientSideWeightedRoundRobin]"
+            ".weight_update_period.seconds "
+            "error:value must be in the range [0, 315576000000]]")
+      << result.status();
+}
+
 //
 // RingHash
 //
