@@ -49,6 +49,22 @@ DEFAULTS = {
     'release': 'kDefaultForReleaseOnly',
 }
 
+FINAL_RETURN = {
+    'broken': 'return false;',
+    False: 'return false;',
+    True: 'return true;',
+    'debug': '#ifdef NDEBUG\nreturn false;\n#else\nreturn true;\n#endif',
+    'release': '#ifdef NDEBUG\nreturn true;\n#else\nreturn false;\n#endif',
+}
+
+FINAL_DEFINE = {
+    'broken': None,
+    False: None,
+    True: '#define %s',
+    'debug': '#ifndef NDEBUG\n#define %s\n#endif',
+    'release': '#ifdef NDEBUG\n#define %s\n#endif',
+}
+
 BZL_LIST_FOR_DEFAULTS = {
     'broken': None,
     False: 'off',
@@ -60,6 +76,7 @@ BZL_LIST_FOR_DEFAULTS = {
 error = False
 today = datetime.date.today()
 two_quarters_from_now = today + datetime.timedelta(days=180)
+experiment_annotation = 'gRPC experiments:'
 for attr in attrs:
     if 'name' not in attr:
         print("experiment with no name: %r" % attr)
@@ -92,6 +109,11 @@ for attr in attrs:
                   (attr['name'], attr['expiry']))
             print("expiry should be no more than two quarters from now")
             error = True
+        experiment_annotation += attr['name'] + ':0,'
+
+if len(experiment_annotation) > 2000:
+    print("comma-delimited string of experiments is too long")
+    error = True
 
 if error:
     sys.exit(1)
@@ -143,22 +165,49 @@ def put_copyright(file, prefix):
         put_banner([file], [line[2:].rstrip() for line in copyright], prefix)
 
 
-EXPERIMENT_METADATA = """struct ExperimentMetadata {
-  const char* name;
-  const char* description;
-  bool default_value;
-};"""
+WTF = """
+This file contains the autogenerated parts of the experiments API.
+
+It generates two symbols for each experiment.
+
+For the experiment named new_car_project, it generates:
+
+- a function IsNewCarProjectEnabled() that returns true if the experiment
+  should be enabled at runtime.
+
+- a macro GRPC_EXPERIMENT_IS_INCLUDED_NEW_CAR_PROJECT that is defined if the
+  experiment *could* be enabled at runtime.
+
+The function is used to determine whether to run the experiment or
+non-experiment code path.
+
+If the experiment brings significant bloat, the macro can be used to avoid
+including the experiment code path in the binary for binaries that are size
+sensitive.
+
+By default that includes our iOS and Android builds.
+
+Finally, a small array is included that contains the metadata for each
+experiment.
+
+A macro, GRPC_EXPERIMENTS_ARE_FINAL, controls whether we fix experiment
+configuration at build time (if it's defined) or allow it to be tuned at
+runtime (if it's disabled).
+
+If you are using the Bazel build system, that macro can be configured with
+--define=grpc_experiments_are_final=true
+"""
 
 with open('src/core/lib/experiments/experiments.h', 'w') as H:
     put_copyright(H, "//")
 
     put_banner(
         [H],
-        ["Automatically generated by tools/codegen/core/gen_experiments.py"],
-        "//")
+        ["Automatically generated by tools/codegen/core/gen_experiments.py"] +
+        WTF.splitlines(), "//")
 
-    print("#ifndef GRPC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
-    print("#define GRPC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
+    print("#ifndef GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
+    print("#define GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
     print(file=H)
     print("#include <grpc/support/port_platform.h>", file=H)
     print(file=H)
@@ -167,21 +216,33 @@ with open('src/core/lib/experiments/experiments.h', 'w') as H:
     print(file=H)
     print("namespace grpc_core {", file=H)
     print(file=H)
+    print("#ifdef GRPC_EXPERIMENTS_ARE_FINAL", file=H)
     for i, attr in enumerate(attrs):
+        define_fmt = FINAL_DEFINE[attr['default']]
+        if define_fmt:
+            print(define_fmt %
+                  ("GRPC_EXPERIMENT_IS_INCLUDED_%s" % attr['name'].upper()),
+                  file=H)
+        print("inline bool Is%sEnabled() { %s }" %
+              (snake_to_pascal(attr['name']), FINAL_RETURN[attr['default']]),
+              file=H)
+    print("#else", file=H)
+    for i, attr in enumerate(attrs):
+        print("#define GRPC_EXPERIMENT_IS_INCLUDED_%s" % attr['name'].upper(),
+              file=H)
         print("inline bool Is%sEnabled() { return IsExperimentEnabled(%d); }" %
               (snake_to_pascal(attr['name']), i),
               file=H)
     print(file=H)
-    print(EXPERIMENT_METADATA, file=H)
-    print(file=H)
     print("constexpr const size_t kNumExperiments = %d;" % len(attrs), file=H)
     print(
         "extern const ExperimentMetadata g_experiment_metadata[kNumExperiments];",
         file=H)
     print(file=H)
+    print("#endif", file=H)
     print("}  // namespace grpc_core", file=H)
     print(file=H)
-    print("#endif  // GRPC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
+    print("#endif  // GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H", file=H)
 
 with open('src/core/lib/experiments/experiments.cc', 'w') as C:
     put_copyright(C, "//")
@@ -194,6 +255,7 @@ with open('src/core/lib/experiments/experiments.cc', 'w') as C:
     print("#include <grpc/support/port_platform.h>", file=C)
     print("#include \"src/core/lib/experiments/experiments.h\"", file=C)
     print(file=C)
+    print("#ifndef GRPC_EXPERIMENTS_ARE_FINAL", file=C)
     print("namespace {", file=C)
     for attr in attrs:
         print("const char* const description_%s = %s;" %
@@ -224,6 +286,7 @@ with open('src/core/lib/experiments/experiments.cc', 'w') as C:
     print("};", file=C)
     print(file=C)
     print("}  // namespace grpc_core", file=C)
+    print("#endif", file=C)
 
 bzl_to_tags_to_experiments = dict((key, collections.defaultdict(list))
                                   for key in BZL_LIST_FOR_DEFAULTS.keys()
