@@ -16,7 +16,7 @@
 set -eo pipefail
 
 # Constants
-readonly PYTHON_VERSION="${PYTHON_VERSION:-3.7}"
+readonly PYTHON_VERSION="${PYTHON_VERSION:-3.9}"
 # Test driver
 readonly TEST_DRIVER_REPO_NAME="grpc"
 readonly TEST_DRIVER_REPO_URL="https://github.com/${TEST_DRIVER_REPO_OWNER:-grpc}/grpc.git"
@@ -42,11 +42,11 @@ readonly GKE_CLUSTER_PSM_BASIC="psm-basic"
 activate_gke_cluster() {
   case $1 in
     GKE_CLUSTER_PSM_LB)
-      GKE_CLUSTER_NAME="interop-test-psm-lb-v1-us-central1-a"
+      GKE_CLUSTER_NAME="psm-interop-lb-primary"
       GKE_CLUSTER_ZONE="us-central1-a"
       ;;
     GKE_CLUSTER_PSM_SECURITY)
-      GKE_CLUSTER_NAME="interop-test-psm-sec-v2-us-central1-a"
+      GKE_CLUSTER_NAME="psm-interop-security"
       GKE_CLUSTER_ZONE="us-central1-a"
       ;;
     GKE_CLUSTER_PSM_BASIC)
@@ -75,11 +75,7 @@ activate_gke_cluster() {
 activate_secondary_gke_cluster() {
   case $1 in
     GKE_CLUSTER_PSM_LB)
-      SECONDARY_GKE_CLUSTER_NAME="interop-test-psm-lb-v1-us-west1-b"
-      SECONDARY_GKE_CLUSTER_ZONE="us-west1-b"
-      ;;
-    GKE_CLUSTER_PSM_SECURITY)
-      SECONDARY_GKE_CLUSTER_NAME="interop-test-psm-sec-v2-us-west1-b"
+      SECONDARY_GKE_CLUSTER_NAME="psm-interop-lb-secondary"
       SECONDARY_GKE_CLUSTER_ZONE="us-west1-b"
       ;;
     *)
@@ -220,7 +216,7 @@ test_driver_get_source() {
 }
 
 #######################################
-# Install Python modules from required in $TEST_DRIVER_FULL_DIR/requirements.txt
+# Install Python modules from required in $TEST_DRIVER_FULL_DIR/requirements.lock
 # to Python virtual environment. Creates and activates Python venv if necessary.
 # Globals:
 #   TEST_DRIVER_FULL_DIR
@@ -249,7 +245,7 @@ test_driver_pip_install() {
     source "${venv_dir}/bin/activate"
   fi
 
-  python3 -m pip install -r requirements.txt
+  python3 -m pip install -r requirements.lock
   echo "Installed Python packages:"
   python3 -m pip list
 }
@@ -365,7 +361,7 @@ kokoro_setup_python_virtual_environment() {
   eval "$(pyenv virtualenv-init -)"
   py_latest_patch="$(pyenv versions --bare --skip-aliases | grep -E "^${PYTHON_VERSION}\.[0-9]{1,2}$" | sort --version-sort | tail -n 1)"
   echo "Activating python ${py_latest_patch} virtual environment"
-  pyenv virtualenv --no-pip "${py_latest_patch}" k8s_xds_test_runner
+  pyenv virtualenv --without-pip "${py_latest_patch}" k8s_xds_test_runner
   pyenv local k8s_xds_test_runner
   pyenv activate k8s_xds_test_runner
   python3 -m ensurepip
